@@ -615,10 +615,13 @@ class Manager(object):
             shutil.copy2(aggregate_file, agg_file_their_orig)
 
         try:
-            source.clone.remote().pull()
+            source.clone.git.fetch('--recurse-submodules=yes')
+            source.clone.git.pull()
+            source.clone.git.submodule('sync', '--recursive')
+            source.clone.git.submodule('update', '--recursive', '--init')
         except git.exc.GitCommandError as error:
             LOG.error('failed to pull source %s: %s', name, error)
-            return 'failed to pull from remote source'
+            return 'failed to pull from remote source: {}'.format(error)
 
         if os.path.isfile(agg_file_ours):
             if os.path.isfile(aggregate_file):
@@ -672,7 +675,7 @@ class Manager(object):
                     else:
                         version = 'master'
 
-                    clone.git.checkout(version)
+                    _git_checkout(clone, version)
 
                     metadata_file = os.path.join(
                         clone.working_dir, METADATA_FILENAME)
@@ -730,7 +733,7 @@ class Manager(object):
             LOG.debug('fetch package %s', ipkg.package.qualified_name())
 
             try:
-                clone.remote().fetch()
+                clone.git.fetch('--recurse-submodules=yes')
             except git.exc.GitCommandError as error:
                 LOG.warn('failed to fetch package %s: %s',
                          ipkg.package.qualified_name(), error)
@@ -780,7 +783,9 @@ class Manager(object):
             version_tags = _get_version_tags(clone)
             return self._install(ipkg.package, version_tags[-1])
         elif ipkg.status.tracking_method == TRACKING_METHOD_BRANCH:
-            clone.remote().pull()
+            clone.git.pull()
+            clone.git.submodule('sync', '--recursive')
+            clone.git.submodule('update', '--recursive', '--init')
             return self._install(ipkg.package, ipkg.status.current_version)
         elif ipkg.status.tracking_method == TRACKING_METHOD_COMMIT:
             # The above check for whether the installed package is outdated
@@ -1000,6 +1005,7 @@ class Manager(object):
             along with the package info object retrieved by inspecting git repo
             contained in the bundle.
         """
+        LOG.debug('getting bundle info for file "%s"', bundle_file)
         bundle_dir = os.path.join(self.scratch_dir, 'bundle')
         delete_path(bundle_dir)
         make_dir(bundle_dir)
@@ -1028,6 +1034,7 @@ class Manager(object):
             package = Package(git_url=git_url, name=git_url.split('/')[-1],
                               canonical=True)
             pkg_path = os.path.join(bundle_dir, package.name)
+            LOG.debug('getting info for bundled package "%s"', package.name)
             pkg_info = self.info(pkg_path, version=version,
                                  prefer_installed=False)
             infos.append((git_url, version, pkg_info))
@@ -1136,7 +1143,7 @@ class Manager(object):
                 version = 'master'
 
         try:
-            clone.git.checkout(version)
+            _git_checkout(clone, version)
         except git.exc.GitCommandError:
             reason = 'no such commit, branch, or version tag: "{}"'.format(
                 version)
@@ -1675,6 +1682,7 @@ class Manager(object):
             str: an empty string if the operation was successful, else an error
             message indicated what went wrong.
         """
+        LOG.debug('unbundle "%s"', bundle_file)
         bundle_dir = os.path.join(self.scratch_dir, 'bundle')
         delete_path(bundle_dir)
         make_dir(bundle_dir)
@@ -1704,6 +1712,7 @@ class Manager(object):
             delete_path(clonepath)
             shutil.move(os.path.join(bundle_dir, package.name), clonepath)
 
+            LOG.debug('unbundle installing "%s"', package.name)
             error = self._install(package, version, use_existing_clone=True)
 
             if error:
@@ -1783,7 +1792,7 @@ class Manager(object):
                         False, test_dir)
 
             try:
-                clone.git.checkout(version)
+                _git_checkout(clone, version)
             except git.exc.GitCommandError as error:
                 LOG.warning('failed to checkout git repo version: %s', error)
                 return (str.format('failed to checkout {} of {}',
@@ -2114,7 +2123,7 @@ class Manager(object):
 
         status.current_version = version
         status.current_hash = _get_hash(clone, version, status.tracking_method)
-        clone.git.checkout(version)
+        _git_checkout(clone, version)
         status.is_outdated = _is_clone_outdated(
             clone, version, status.tracking_method)
 
@@ -2295,6 +2304,22 @@ def _copy_package_dir(package, dirname, src, dst, scratch_dir):
     return ''
 
 
+def _git_checkout(clone, version):
+    """Checkout a version of a git repo along with any associated submodules.
+
+    Args:
+        clone (git.Repo): the git clone on which to operate
+
+        version (str): the branch, tag, or commit to checkout
+
+    Raises:
+        git.exc.GitCommandError: if the git repo is invalid
+    """
+    clone.git.checkout(version)
+    clone.git.submodule('sync', '--recursive')
+    clone.git.submodule('update', '--recursive', '--init')
+
+
 def _create_readme(file_path):
     if os.path.exists(file_path):
         return
@@ -2360,10 +2385,12 @@ def _info_from_clone(clone, package, status, version):
     if invalid_reason:
         return PackageInfo(package=package, invalid_reason=invalid_reason,
                            status=status, versions=versions,
-                           metadata_version=version, version_type=version_type)
+                           metadata_version=version, version_type=version_type,
+                           metadata_file=metadata_file)
 
     metadata = _get_package_metadata(metadata_parser)
 
     return PackageInfo(package=package, invalid_reason=invalid_reason,
                        status=status, metadata=metadata, versions=versions,
-                       metadata_version=version, version_type=version_type)
+                       metadata_version=version, version_type=version_type,
+                       metadata_file=metadata_file)
